//
// Copyright 2023 Pixar
//
// Licensed under the terms set forth in the LICENSE.txt file available at
// https://openusd.org/license.
//
////////////////////////////////////////////////////////////////////////

/* ************************************************************************** */
/* **                                                                      ** */
/* ** This file is generated by a script.                                  ** */
/* **                                                                      ** */
/* ** Do not edit it directly (unless it is within a CUSTOM CODE section)! ** */
/* ** Edit hdSchemaDefs.py instead to make changes.                        ** */
/* **                                                                      ** */
/* ************************************************************************** */

#ifndef PXR_IMAGING_HD_EXT_COMPUTATION_PRIMVAR_SCHEMA_H
#define PXR_IMAGING_HD_EXT_COMPUTATION_PRIMVAR_SCHEMA_H

/// \file

#include "pxr/imaging/hd/api.h"
#include "pxr/imaging/hd/primvarSchema.h"

#include "pxr/imaging/hd/schema.h"

// --(BEGIN CUSTOM CODE: Includes)--
// --(END CUSTOM CODE: Includes)--

PXR_NAMESPACE_OPEN_SCOPE

// --(BEGIN CUSTOM CODE: Declares)--
// --(END CUSTOM CODE: Declares)--

#define HD_EXT_COMPUTATION_PRIMVAR_SCHEMA_TOKENS \
    (interpolation) \
    (role) \
    (sourceComputation) \
    (sourceComputationOutputName) \
    (valueType) \

TF_DECLARE_PUBLIC_TOKENS(HdExtComputationPrimvarSchemaTokens, HD_API,
    HD_EXT_COMPUTATION_PRIMVAR_SCHEMA_TOKENS);

//-----------------------------------------------------------------------------


class HdExtComputationPrimvarSchema : public HdSchema
{
public:
    /// \name Schema retrieval
    /// @{

    HdExtComputationPrimvarSchema(HdContainerDataSourceHandle container)
      : HdSchema(container) {}

    /// @}

// --(BEGIN CUSTOM CODE: Schema Methods)--

    // these return statically allocated instances for the common cases.
    static HdTokenDataSourceHandle BuildInterpolationDataSource(
            TfToken interpolation) {
        return HdPrimvarSchema::BuildInterpolationDataSource(interpolation);
    }

    static HdTokenDataSourceHandle BuildRoleDataSource(TfToken role) {
        return HdPrimvarSchema::BuildRoleDataSource(role);
    }

// --(END CUSTOM CODE: Schema Methods)--

    /// \name Member accessor
    /// @{

    HD_API
    HdTokenDataSourceHandle GetInterpolation() const;

    HD_API
    HdTokenDataSourceHandle GetRole() const;

    HD_API
    HdPathDataSourceHandle GetSourceComputation() const;

    HD_API
    HdTokenDataSourceHandle GetSourceComputationOutputName() const;

    HD_API
    HdTupleTypeDataSourceHandle GetValueType() const; 

    /// @} 

    /// \name Schema construction
    /// @{

    /// \deprecated Use Builder instead.
    ///
    /// Builds a container data source which includes the provided child data
    /// sources. Parameters with nullptr values are excluded. This is a
    /// low-level interface. For cases in which it's desired to define
    /// the container with a sparse set of child fields, the Builder class
    /// is often more convenient and readable.
    HD_API
    static HdContainerDataSourceHandle
    BuildRetained(
        const HdTokenDataSourceHandle &interpolation,
        const HdTokenDataSourceHandle &role,
        const HdPathDataSourceHandle &sourceComputation,
        const HdTokenDataSourceHandle &sourceComputationOutputName,
        const HdTupleTypeDataSourceHandle &valueType
    );

    /// \class HdExtComputationPrimvarSchema::Builder
    /// 
    /// Utility class for setting sparse sets of child data source fields to be
    /// filled as arguments into BuildRetained. Because all setter methods
    /// return a reference to the instance, this can be used in the "builder
    /// pattern" form.
    class Builder
    {
    public:
        HD_API
        Builder &SetInterpolation(
            const HdTokenDataSourceHandle &interpolation);
        HD_API
        Builder &SetRole(
            const HdTokenDataSourceHandle &role);
        HD_API
        Builder &SetSourceComputation(
            const HdPathDataSourceHandle &sourceComputation);
        HD_API
        Builder &SetSourceComputationOutputName(
            const HdTokenDataSourceHandle &sourceComputationOutputName);
        HD_API
        Builder &SetValueType(
            const HdTupleTypeDataSourceHandle &valueType);

        /// Returns a container data source containing the members set thus far.
        HD_API
        HdContainerDataSourceHandle Build();

    private:
        HdTokenDataSourceHandle _interpolation;
        HdTokenDataSourceHandle _role;
        HdPathDataSourceHandle _sourceComputation;
        HdTokenDataSourceHandle _sourceComputationOutputName;
        HdTupleTypeDataSourceHandle _valueType;

    };

    /// @}
};

PXR_NAMESPACE_CLOSE_SCOPE

#endif